/*
 * Copyright (C) 2021 Purism SPC
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * Author: Guido Günther <agx@sigxcpu.org>
 */

#include "phosh-wayland.h"

#include <glib.h>

G_BEGIN_DECLS

void phosh_test_head_stub_init        (PhoshWayland *wl);
void phosh_test_head_stub_destroy     (void);

G_END_DECLS
